        subroutine findrays(raydat,ttt,ang,recang,ntpt,nrays,itemplate)
c
c      This subroutine identifies ray arrivals from a raytrace'
c        print *, 'to arrivals saved in a template or reference filename'
c
c     This routine returns 2 or 9 for 'itemplate' depending on if the traced ray 
c     was found in the template or not, respectively.
c
c     The travel time bin (dt=2.5 sec) and source (da1=2.0 deg) and arrival 
c     angle (da2=2.0 deg) bins into which a traced ray must fall to be identified 
c     can be modifed by the user in the source code below.  Note that the number
c     of turning points must be always exactly matched.
c
c     B. Dushaw (dushaw@apl.washington.edu) 
c
c     Eigenray Version 3.4
c     Last updated November 2012

        include 'nrays.h'
        implicit double precision (a-h,o-z)
        double precision raydat(neig,8)

c default is no found ray with a flag of 9.  If a ray is successfully found
c a flag of 2 is returned for itemplate.
        itemplate=9

        do 15 j=1,nrays
                dt=abs(raydat(j,1)-ttt)
                da1=abs(raydat(j,2)-ang)
                da2=abs(raydat(j,3)-recang)
                nd=iabs(int(raydat(j,8))-ntpt)

c     2.5 is 2.5 sec - a parameter to adjust for better tracking...  
c     2.0 is 2.0 deg - a parameter to adjust for better tracking...  
c     No. of turning points (ntpt vs raydat(j,8)) must be the same.
c     If the number of turning points are the same, and the rays have
c     the same sign of angles, then it is most likely the ray we are after.
              if(dt.lt.2.5d0.and.
     1          da1.lt.2.0d0.and.
c     1          da2.lt.3.0d0)then
     1          da2.lt.2.0d0.and.
     1           nd.eq.0)then
c     1           nd.lt.2)then
                   itemplate=2
c flag out the found ray in the template to avoid reproducing it later
                   raydat(j,1)=1.0d9
                   raydat(j,2)=1.0d9
                   raydat(j,3)=1.0d9
                   raydat(j,8)=1.0d9
                   return
              endif
15      continue

        return
        end
