       subroutine odeint(ystart,x1,x2,xp,yp,deltaX,kount,iprof,kmax)
       implicit double precision (a-h,o-z)
c
c MAXSTP is limit for main loop for ray integration over the entire propagation range.
c MAXSTP should be at least range/dxsav or 5000000/500=10000 for
c        5 Mm range saved every 500 m.
c
c KMAXX  is the size of the arrays xp, yp for range and depth of
c        raypath.
c
c kmax is set to zero if raypaths are not to be saved
c
c  Brian Dushaw
c  Applied Physics Laboratory
c  University of Washington
c  Seattle, WA 98105
c  (206) 543-1300
c  dushaw@apl.washington.edu

c  Documentation (e.g., an APL Technical Memorandum) may be found at:
c  http://staff.washington.edu/dushaw/eigenray/index.html
c  http://909ers.apl.washington.edu/twiki/bin/viewauth/Main/EigenRay

c  EIGENRAY Version 3.4
c  September-December 1998
c  Last updated March 2013
c
      PARAMETER (surf=0.002d0,MAXSTP=50000000)
c MAXSTP = 6000000 should handle north pacific rays at 5 Mm range
c for reasonable step size.
      include 'nrays.h'
      DOUBLE PRECISION dydx(4),y(4),ytemp(4)
      DOUBLE PRECISION ystart(4)
      double precision xp(KMAXX),yp(2,KMAXX),deltaX
      integer kount,iprof,kmax
      INTEGER i,nstp,itmp

      include 'nssp.h'
      include 'table.h'
      include 'path.h'
      include 'nbath.h'
      include 'bathy.h'
      include 'constants.h'

      iraycount=iraycount+1
      xsav=zero
      iprof=1
c iprof is the index for sound speed profile range
      ibdex=1
c ibdex is the index for bathymetry range

      deltaX=zero
c flags for surface reflections, bottom reflections, and counting
c bottom reflections
      iturnflag=0
      ibotflag=0
      ibotbonk=0
      xprevious=x1
      igetout=0
      hstart=zero

      ds_c2=zero

      x=x1
      xmark=x+dxsav

      kount=0
      do 11 i=1,4
        y(i)=ystart(i)
        ytemp(i)=0.0
11    continue

      if (kmax.gt.0) xsav=x-two*dxsav

      do 16 nstp=1,MAXSTP

c steps is a table set up in stps.f
   
      itmp=int(xinv_depth_inc*y(3))
      if(0.le.abs(itmp).and.abs(itmp).le.Ndepmax) then
        h=steps(itmp)
      else
        print *,'STEPS PROBLEM: ',abs(itmp),itmp,y(3)
        goto 17
      endif

      call derivs(y,dydx,deltaX,iprof)

c decide if have stepped enough that a ray point should be saved.
        if(kmax.gt.0)then
          if(dabs(x-xsav).ge.dabs(dxsav)-1.d-10) then
c  (1.d-10...I worry so about round off error)
            if(kount.lt.kmax-1)then
              kount=kount+1
              if(kount.gt.KMAXX)goto17
              xp(kount)=x

c only need to save depth and ds_c2 along the raypath.
                yp(1,kount)=y(3)
                yp(2,kount)=ds_c2
                ds_c2=zero
              xsav=x
            endif
          endif
        endif

c  surface reflection check.  If will be within 0.02 m, call it a reflection.
c  pretend surface is at 0.02 m for security....  depth z>0 throughout.

        if((y(3)+h*dydx(3)).le.surf.and.(x-xprevious).gt.100.d0)then
c the next increment of h will put the ray above the surface.
c flag it and select an h to give a ray arriving right at the surface.
c xprevious is to guard against rays which graze the surface (i.e. if you've
c reflected in the previous step, perhaps the ray has not gotten far enough 
c away from the surface yet.)
          iturnflag=1
          if(dydx(3).ne.zero) then
             h=(surf-y(3))/dydx(3)
          else
c in this case the ray slope is zero AND the ray is at zero depth.
             h=5.d0
             y(3)=surf
          endif
          xprevious=x
        endif

c  bottom reflection check.
      if(ibot.eq.1)then
c NB: everything is positive down
c      temp1=y(3)-btab(2,ibdex)-(x-btab(1,ibdex))*btab(3,ibdex)
      temp1=-dabs(y(3)-x*btab(3,ibdex)-btab(4,ibdex))
c pretend like the incoming ray is always above the bottom...

      temp=h*(dydx(3) - btab(3,ibdex)) + temp1
        if(temp.ge.zero.and.temp1.ne.temp.and.dydx(3).ne.zero)then
c the next step will be below the bottom, solve for h to put
c us right at the bottom in preparation for reflection.
          ibotflag=1
          ibotbonk=ibotbonk+1
          igetout=0
          if(ibotbonk.gt.MBONK) goto 17
c drop ray if # bounces greater than MBONK

          h=(temp-temp1)/(dydx(3) - btab(3,ibdex))

        endif
c save y(i) in preparation for the iteration to exactly the bottom
c to be done next.
          do i=1,4
          ytemp(i)=y(i)
          enddo
      endif
 
c If below is true, we are near the end point of the integration, i.e.
c near the receiver range.  Set h to arrive at exactly at the receiver
c range.  (x+h) < x2 and (x+h) > x1 always.     
        if ((x+h-x2).ge.zero) h=x2-x

c Now, a very sophisticated range dependence.....
        if(x.gt.range(iprof+1)) iprof=iprof+1
c We do not want to increment iprof in the middle of a surface reflection.

c deltaX is used in derivs and getc to get deltaX*dc/dr, and deltaX*d(dc/dz)/dr
c it appears not to matter if deltaX or deltaX+h is used.
        deltaX=x-range(iprof)

c the lines below result in step sizes such that the ray paths are saved
c at exactly the user specified increments
      if((x+h).gt.xmark)then 
        h=xmark-x
        xmark=xmark+dxsav
        ibotflag=0
      endif

c safety check....
      if(h.le.zero) h=0.001d0

c############################################################################
c rk4 is the 4th order R-K integration step.
c y(i) are replaced by their new values.  dydx are not used again.

 87    hstart=h  
       hc=zero
       istepout=0
 88      call rk4(y,dydx,h,deltaX,iprof)

         if(y(3).gt.XMAXDEPTH) goto 17
c y(1) is cos(angle); it should always be near to 1, but it occasionally
c wanders off, perhaps because of a glitch in sound speed - abort if the 
c angle gets too steep.
        if(dabs(y(1)).lt.0.2)goto 17

c The stepsize h may have changed in rk4 on account of epsilon.
c Put a loop in here to accumulate h's until we get back to the original step size.
c Since h is scaled down by 0.5, this loop will usually take 2, 4, 8, etc. times.
c We do want the overall step size to be the original hstart.
       if(dabs(h-hstart).gt.1.d-6) then
         istepout=istepout+1
c y(3) is the depth of the ray - skip the ray if it goes deeper than XMAXDEPTH
c         if(y(3).gt.XMAXDEPTH.or.istepout.gt.MAXISTP) then
c Changed from a fixed value to one that is double the initial stepsize.  
c The idea being that if h gets to be 1 m, the maximum steps will be hstart.
         if(y(3).gt.XMAXDEPTH.or.istepout.gt.(floor(hstart)*2)) then
            goto 17
         endif
         hc=hc+h
         if(dabs(hc-hstart).lt.1.d-6)then
            h=hc
            goto 89
         endif
         call derivs(y,dydx,deltaX,iprof)
         goto 88
       endif
 89    continue

c y(1) is cos(angle); it should always be less than 1, but it occasionally
c gets greater than 1 - it seems this occurs because of round off error in
c the rk4 integration, particularly near the turning points
         if(y(1).gt.one) then
            y(1)=one
            y(2)=zero
         endif

c y(1) is cos(angle); it should always be near to 1, but it occasionally
c wanders off, perhaps because of a glitch in sound speed - abort if the 
c angle gets too steep.
        if(dabs(y(1)).lt.0.2)goto 17

c y(2) is sin(angle); if it gets bigger than 1 then something has happened to
c the ray, and so we punt and get out.  This is probably a problem with
c the sound speed profile near the surface catching the ray just in
c the wrong way.  The integration spirals out of control after this.
        if(dabs(y(2)).gt.one)goto 17

c y(3) is the depth of the ray - skip the ray if it goes deeper than XMAXDEPTH
        if(y(3).gt.XMAXDEPTH) goto 17

c ztemp is the depth of the bottom at the range x+h. btab(3,ibdex) is
c the slope, and btab(4,ibdex) is the intercept.
      if(ibot.eq.1)then

        if((x+h).gt.btab(1,ibdex+1)) then
           ztemp=(x+h)*btab(3,ibdex+1) + btab(4,ibdex+1)
        else 
           ztemp=(x+h)*btab(3,ibdex) + btab(4,ibdex)
        endif
c  ztemp is the depth of the bottom at the range x+h

c towards the bottom the step sizes are rather large and the approximation
c above to solve for h doesn't work too well.  We need to iterate towards
c the proper h to get the ray right at the bottom (btol meters in this case).
c If we overshoot we will back up a lot, and then creep forward...
      if(ibotflag.eq.1.or.((y(3)-ztemp).ge.zero)) then

c first guard against rays which reach the bottom without first being detected above
         if(((y(3)-ztemp).ge.zero).and.ibotflag.eq.0)then
             ibotflag=1
             igetout=0
         endif

        if((y(3)-ztemp).gt.btol)then
c the ray has overshot bottom by more than tolerance; need to use a smaller step size
c Reset and set a smaller stepsize, repeat
          do i=1,4 
             y(i)=ytemp(i) 
          enddo
          igetout=igetout+1
          if(igetout.gt.15)then
             igetout=0
             goto 44
          endif
          h=0.8d0*h
          goto 87
        elseif((y(3)-ztemp).lt.-btol)then
c the ray has not yet reached the bottom, increase h slowly to move the
c ray slowly towards the bottom.
c Reset and set a larger stepsize, repeat
          do i=1,4
            y(i)=ytemp(i)
          enddo
          igetout=igetout+1
          if(igetout.gt.15)then
             igetout=0
             goto 44
          endif
          h=1.05d0*h
          goto 87
        endif
      endif
      endif
 44   x=x+h
c############################################################################

c we have taken a range step of increment h now.
       if(kmax.gt.0.and.iraysave.eq.1) then
c We need to save the ray path and we need ds/c^2
c This section has an expensive division to do; only do this
c step if you really need it.
         call getc(y(3),c,deltaX,iprof)
         ds_c2 = ds_c2 + h/(c*c*y(1))
c   ds/c^2 = dx/(c^2 cos(theta)), and cos(theta) > 0 always.
       endif

c  Continue surface reflection check.  
c  Flip sign of angle cos(theta)=>cos(theta)
c  Flip sign of angle sin(theta)=>-sin(theta)
c  Sign of dydx slopes are irrelevant; they are recalculated at the top of
c  the do loop.
       if(iturnflag.eq.1)then
          iturnflag=0
          y(2)=-y(2)

c save the ray point, if need be...regardless of how far we have gone.
          if(kmax.gt.0)then
            if(kount.lt.kmax-1)then
              kount=kount+1
              if(kount.gt.KMAXX)goto17
              xp(kount)=x
              yp(1,kount)=y(3)
              yp(2,kount)=ds_c2
              ds_c2=zero
            endif
          endif
       endif
c  end of surface reflection check

       if(ibot.eq.1) then
115        if(x.gt.btab(1,ibdex+1))then
                ibdex=ibdex + 1
                goto 115
           endif
c continue bottom reflection check - reflect the ray.
           if(ibotflag.eq.1) then
              ibotflag=0
c go to Z positive up sign convention for slope and ray angle to avoid confusion.
              tanb=-btab(3,ibdex)
              sinb=tanb/dsqrt(one+tanb*tanb)
              cosb=sqrt(one-sinb*sinb)       
c vector along bottom is b=( cosb, sinb)
c normal to bottom is    s=(-sinb, cosb), which is directed waterward 
c approaching ray vector is n1=(y(1),-y(2)); an angle sign change because of code convention
c new ray vector is n2=n1-2*(n1.s)s
c n1.s is the dot product (dp) of n1 and s.
c              xn=one/dsqrt(y(1)*y(1)+y(2)*y(2))
c              y(1)=y(1)*xn
c              y(2)=-y(2)*xn
              y(2)=-y(2)
              dp=two*(-y(1)*sinb + y(2)*cosb)
              y(1) = y(1) + dp*sinb
              y(2) = y(2) - dp*cosb
c normalization not really needed - put in for safety
c              xn=one/dsqrt(y(1)*y(1)+y(2)*y(2))
c              y(1)=y(1)*xn
c              y(2)=-y(2)*xn
               y(2)=-y(2)
c              new_angle = atan2(y(2),y(1))

c save the ray point, if need be...regardless of how far we have gone.
            if(kmax.gt.0)then
              if(kount.lt.kmax-1)then
                kount=kount+1
                if(kount.gt.KMAXX)goto17
                  xp(kount)=x
                  yp(1,kount)=y(3)
                  yp(2,kount)=ds_c2
                  ds_c2=zero
                endif
              endif
            endif
       endif

        if((x-x2).gt.-1.0d-4)then
c we are done with this path; save the end point and the path point
c and return.
          do 14 i=1,4
            ystart(i)=y(i)
14        continue
          if(kmax.ne.0)then
            kount=kount+1
              if(kount.gt.KMAXX)goto17
            xp(kount)=x
            yp(1,kount)=y(3)
            yp(2,kount)=ds_c2
            ds_c2=zero
          endif
          return
        endif

c Continue main loop of ray path integration.
16    continue

c      print *,x,h,(180.d0/3.14159d0)*dasin(y(2)),y(3),y(4)
      print *, 'In dodeint: ignore this or increase step size or MAXSTP 
     *'
      print *,'Terminating the calculation of this ray'
      ystart(3)=-1.d0
      return
c
c flag ending depth as less than zero to know that ray was dropped
17    if (ibotbonk.le.MBONK) then
      print *,'Problem with ray in dodeint. Some rays just wont make it.
     * A Clue:'
c Dropping a ray for excessive bottom bounces is normal, we hope
c So take no note of it.

      if (kount.gt.KMAXX)
     *print *,'kount=',kount,' Max kount=',KMAXX, 
     *'( Array size for ray)'
c      print *,'kount=',kount,' Max kount=',KMAXX
      
c      if (ibotbonk.gt.MBONK)
c     *print *,'ibotbonk=',ibotbonk,' Max ibotbonk=',MBONK, 
c     *' (Maximum number of bottom bounces)'
c      print *,'ibotbonk=',ibotbonk,' Max ibotbonk=',MBONK

      if (y(3).gt.XMAXDEPTH)
     *print *,'y(3)=',y(3),' Max y(3) (ray depth)=',XMAXDEPTH,
     *' (Maximum allowed ray depth)'
c      print *,'y(3)=',y(3),' Max y(3) (ray depth)=',XMAXDEPTH

      if (istepout.gt.(floor(hstart)*2))
     *print *,'istepout=',istepout,' Max istepout=',(floor(hstart)*2),
     *' (No convergence/progress in RK step)'
c      print *,'istepout=',istepout,' Max istepout=',(floor(hstart)*2)

      if (dabs(y(2)).gt.one)  
c y(2) is sine(theta); should always be less than or equal to one.
     *print *,'dabs(y(2))=',dabs(y(2)),' Max dabs(y(2))=',one,
     *' (Numerical problem; sine is greater than one!)'
c      print *,'dabs(y(2))=',dabs(y(2)),' Max dabs(y(2)) (cos(x))=',one
c y(1) is cosine(theta); should always be near to the value of one -
c rays are almost horizontal.

      if (dabs(y(2)-ystart(2)).lt.1.0d-08)  
     *print *,'No change in ray initial angle...???'

      if (dabs(y(1)).lt.0.2)  
     *print *,'dabs(y(1))=',dabs(y(1)),' Min dabs(y(1))=',0.2,
     *' (Numerical problem; cosine is too small!)'

      endif

c Flag to note the failed ray; rays should never go to negative depths
c (within numerical error...)
      ystart(3)=-1.d0
      return
      END
