      program driver
c
c This program reads in data and otherwise sets up parameters to call
c subroutine eig.  This program is to replace the original "eigenray" code,
c with a more modular subroutine approach.
c
c  Brian Dushaw
c  Applied Physics Laboratory 
c  University of Washington
c  Seattle, WA 98105
c  (206) 543-1300
c  dushaw@apl.washington.edu

c  Documentation (e.g., an APL Technical Memorandum) may be found at:
c  http://staff.washington.edu/dushaw/eigenray/index.html
c  http://909ers.apl.washington.edu/twiki/bin/viewauth/Main/EigenRay

c  EIGENRAY Version 3.4
c  September-December 1998
c  Last updated November 2012

c Historical Quote (from Moler and Solomon, 1970) "The complete program is not
c complicated.  It requires less than 400 FORTRAN statements, including the 
c differential equation solver, but not including the plotting system.  The solution
c of a problem [like finding eigenrays at 24 mile range] requires only a minute
c or two of elapsed time at a teletype and costs less than $10.00 (at the
c University of Michigan's noncommerical rates)."

      implicit double precision (a-h,o-z)
c We don't want the driver program to have to handle all the tables.  Those 
c includes have large common blocks used for the ray tracing, but we're better 
c off avoiding them in the driver routine.  
c The tables are filled in the eig subroutine.
      include 'nbath.h'
      include 'nssp.h'
      include 'nrays.h'
      include 'constants.h'

c Parameters are defined in the include files above; we define them separately 
c here to be able to leave out the include files from the driver program.  
c The parameters are part of a common block elsewhere.
      
      double precision ssp(NZSSP,Nprofmax),zz(NZSSP),range(Nprofmax)
      double precision s(4),trk(neig,8)
      double precision bb(2,MBATH)
      double precision par(25),xinfo(9,MAXRAYS)
      double precision rpath(3,KMAXX,neig)
      integer iprint
      character*80 name1,name2,name3

c Set iprint=1 to have the program write out all the "Input" statements
      iprint=0

c Zero out xinfo for safety.
      do k=1,MAXRAYS
         do j=1,9
             xinfo(j,k)=zero
         enddo
      enddo
c Start ray count with zero.
      iraycount=0

      npeig=KMAXX

      if(iprint.eq.1) 
     1 print *,'Input starting,ending values of ray angle (degrees):'
      read(5,*) ang1, ang2
      if(iprint.eq.1) print *
      if(iprint.eq.1) 
     1 print *,'Input number of rays for time front sketch:'
      read(5,*)numrays
      if(iprint.eq.1) print *
      if(iprint.eq.1) 
     1 print *,'Input source and receiver depths (meters):'
      read(5,*) z0,zr
      if(iprint.eq.1) print *
      if(iprint.eq.1) print *, 'Input the path range (in meters):'
      read(5,*) r2
      if(iprint.eq.1) print *
      if(iprint.eq.1) 
     1 print *, 'Input the range increment for output ray paths:'
      read(5,*) dxsav
      if(iprint.eq.1) print *
      if(iprint.eq.1) 
     1 print *,'Input tolerance for missing the receiver depth (10 m?),'
      if(iprint.eq.1) 
     1 print *,'and maximum miss you can live with (100 m?):'
      read(5,*) zmiss,zmiss_max
      if(iprint.eq.1) print *
      if(iprint.eq.1) print *,'Save the eigenray paths? 0=no, 1=yes'
      read(5,*)iraysave
      if(iprint.eq.1) print *
      if(iprint.eq.1) 
     1 print *,'Eigenrays or Time Front? 0=eigenrays, 1=timefront'
      read(5,*)itimefront
      if(iprint.eq.1) print *
      if(iraysave.eq.1.and.itimefront.eq.1)then
        print *,'************************************************'
        print *,'************************************************'
        print *,'WARNING!! You are saving the timefront raypaths'
        print *,'STOPPING for now.  Develop eigenray some more to'
        print *,'let this happen.'
        print *,'************************************************'
        print *,'************************************************'
c        stop
      endif
      if(iprint.eq.1) print *
      if(iprint.eq.1) print *,'Enable ocean bottom? 0=no, 1=yes'
      read(5,*) ibot
      if(iprint.eq.1) print *
      if(iprint.eq.1) print *,'Input travel time offset (seconds):'
      read(5,*) ttoffset
      if(iprint.eq.1) print *
      if(iprint.eq.1) 
     1 print *, 'Use a template to ID rays to write out? 0=no, 1=yes'
      read(5,*) itemplate
c      itemplate starts out as a flag with values 0 or 1 as to whether a template is 
c      available or not.  If a template is available it becomes a flag with values 9
c      or 2 depending on whether a traced ray has been identified with one from the template.  

c READING SOUND SPEED FILE
      if(iprint.eq.1) print *
      if(iprint.eq.1) print *,'Input the soundspeed filename'
c Assumes the same format as the RAY *.ssp files.
      read(5,5)name1
 5     format(a)
      call loadssp(ssp,zz,range,izdex,irdex,name1)
c speed assumes equal zz's and a rectangular block of sound speed values.

c READING STEP SIZE INFORMATION
      if(iprint.eq.1) print *
      if(iprint.eq.1) print *,'Step size definition:'
      if(iprint.eq.1) 
     1print *,'Step size varies linearly with depth in the upper ocean,'
      if(iprint.eq.1) 
     1 print *,'and tanh with depth in the lower ocean.'
      if(iprint.eq.1) print *,
     1'Input depth for transition from linear to tanh, e.g. 1500 (m):'
      read(5,*) s(1)
      if(iprint.eq.1) print *,'Input 3 values of step size:'
      if(iprint.eq.1) 
     1 print *,'smaller is more accurate, but takes longer to compute'
      if(iprint.eq.1) 
     1 print *,'1) surface: should probably be less than 10 (m)'
      if(iprint.eq.1) print *,'2) transition point: 100 or so (m)'
      if(iprint.eq.1) print *,'3) at depth: 500-700 (m) should work'
      read(5,*) s(2), s(3), s(4)

c READING EPSILON
      if(iprint.eq.1) print *
      if(iprint.eq.1) 
     1 print *,'Input epsilon parameter for accuracy, e.g. 1e-5:'
      read(5,*)epsilon
      if(iprint.eq.1) print *

c READING TEMPLATE INFORMATION
      if(iprint.eq.1) 
     1 print *,'Input the template ray.info filename e.g. Save_Ann:'
      if(iprint.eq.1) 
     1 print *,'(Use a dummy name if no template is to be used.)'
      if(iprint.eq.1) print *
c if itemplate=0 then we are going to do nothing, but we want to read
c the filename to ensure that the input file "in.ray" always has the same form.
        read(5,5)name2
        nrays=MAXRAYS
        if(itemplate.eq.1)then
           open(135,file=name2,status='old')
           read(135,*)
c  find number of measured ray arrivals and load in the template data
           do k=1,100
             read(135,*,end=1)index2,(trk(k,j),j=1,8)
           enddo
1          nrays=k-1
           close(135)
        endif

c READING BATHYMETRY INFORMATION
      if(iprint.eq.1) print *,'Input maximum number of bounces:'
      read(5,*) MBONK
      if(iprint.eq.1) print *
      if(iprint.eq.1) 
     1 print *,'Input tolerance for missing ocean bottom:'
      read(5,*) btol
      if(iprint.eq.1) print *
      if(iprint.eq.1) print *,'Input bathymetry filename:'
      read(5,7)name3
7     format(a)
      if(ibot.eq.1) then
        call loadbot(bb,MBATH,numbot,name3)
c  end of ibot.eq.1 
      endif
      if(iprint.eq.1) print *
      if(iprint.eq.1) print *,'Input maximum depth for rays'
      XMAXD=zero
      read(5,*,END = 88) XMAXD
88    continue

      print *
      write(6,*) '%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'
      write(6,8) '                  INPUT SUMMARY'
      write(6,24) 'Source and receiver depths (m):', z0,zr
      write(6,24) 'Range (km):', r2*0.001d0
      write(6,24) 'Start and end angles:', ang1,ang2
      write(6,23) 'Number of angles in ray trace:', numrays
      write(6,24) 'Range increment to save raypaths (m):', dxsav
      write(6,24) 'Tolerance for missing receiver depth (m):', zmiss
      write(6,24) 'But willing to accept a miss depth of:', zmiss_max
      if(iraysave.eq.0)then
           write(6,24) 'Not saving the raypaths.'
           kmax=0
      else
           write(6,24) 'Saving the raypaths.'
      endif
      if(itimefront.eq.0) write(6,24) 'Calculating eigenrays.'
      if(itimefront.eq.1) write(6,24) 'Calculating a timefront.'
      if(ibot.eq.0) write(6,24) 'No bottom.'
      if(ibot.eq.1) write(6,24) 'Including bottom topography.'
      write(6,24) 'Using a travel time offset of (s):', ttoffset
      if(itemplate.eq.0) write(6,24), 'Not using a template for ray ID.'
      if(itemplate.eq.1) write(6,24), 'Using a template for ray ID.'
      write(6,24) 'Step size parameters (m):', (s(k),k=1,4)
      write(6,25) 'Value for epsilon:', epsilon
      write(6,23) 'Maximum number of bounces:', MBONK
      write(6,24) 'Tolerance for missing ocean bottom (m):', btol
      write(6,24) 'Maximum depth for rays (m):', XMAXD
      print *
      write(6,8)'Sound speed filename:', name1
      write(6,8)'Ray template filename:', name2
      write(6,8)'Bathymetry filename:', name3
      write(6,*) '%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%'
  8   format(a42,2x,a)
 23   format(a42,i8)
 24   format(a42,4f8.1)
 25   format(a42,e8.1)

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccc  We have all the data we need - calling eig ccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c initiate parameter variables to keep the variable list manageable
c
c  start angle
      par(1)=ang1
c  end angle
      par(2)=ang2
c  number of rays to trace
      par(3)=numrays
c  source depth
      par(4)=z0
c  receiver depth
      par(5)=zr
c  range
      par(6)=r2

c  timefront or eigenray flag
      par(7)=itimefront
c  yes, no ray tracking flag
      par(8)=itemplate
c  number of rays in template
      par(9)=nrays
c  save ray path flag
      par(10)=iraysave
c  range increment to save ray paths
      par(11)=dxsav
c  tolerance for missing the receiver depth
      par(12)=zmiss
c  gross tolerance for missing the receiver depth
      par(13)=zmiss_max

c  travel time offset
      par(14)=ttoffset
c  criteria for accuracy of path integration (1.e-7 suggested)
      par(15)=epsilon

c  yes, no bottom reflection flag
      par(16)=ibot
c  tolerance for bottom reflection
      par(17)=btol
c  maximum number of bottom reflections allowed before dropping ray
      par(18)=MBONK
c  total number of bathymetry data points
      par(19)=numbot
c  maximum depth for rays
      par(20)=XMAXD
       
c  number of depths in sound speed data
      par(21)=izdex
c  number of profiles in sound speed data
      par(22)=irdex

      call eig(ssp,zz,range,bb,trk,s,par,xinfo,rpath)
c Returns xinfo and rpath:  the information about the timefront or eigenrays.
c Also returns the number of rays found and number of rays traced in par.

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c      number of rays found
       kount=int(par(23)+1.0e-6)
c      total number of rays traced
       iraycount=int(par(24)+1.0e-6)

c  Now write out the results

      open(35,file='ray.data')
      open(36,file='ray.info')

      write(36,107)'%   j     time      src.ang rec.ang bot.turn top.tur
     1n  rec.z    snd.spd  #tp'   
107   format(a) 
       ncount=1
       do k=1,kount
c The ray information...
         icount=int(xinfo(1,k))
         ttt=   xinfo(2,k)
         if ((ttt.gt.1.0)) then
c ttt is zero most of the time if a template is being used.  It will be
c non zero if the ray was identified against one of the template rays.
c ... the k.le.nrays is a kludge for now - somehow with a template 
c k can get to be larger than nrays. ???
           ang=   xinfo(3,k)
           recang=xinfo(4,k)
           ztlow= xinfo(5,k)
           ztup=  xinfo(6,k)
           zrec=  xinfo(7,k)
           crec=  xinfo(8,k)
           ntpt=  int(xinfo(9,k))

          write(36,100)ncount,ttt,ang,recang,ztlow,ztup,zrec,crec,ntpt
100     format(i5,1x,f12.5,2x,f6.2,2x,f6.2,2x,f6.0,2x,f6.0,3x,f9.3,2x,
     1 f7.2,1x,i4)
        

c The ray paths...
      if(iraysave.eq.1)then
        do kk=1,npeig
c below writes out the ray path.
          if (kk.gt.1.and.rpath(1,kk,k).lt.one)goto 99
c           range               depth               ds/c^2
          write(35,35) 
     crpath(1,kk,k), rpath(2,kk,k), rpath(3,kk,k)
35        format(f14.4,2x,f10.4,2x,f12.9)
        enddo
99      write(35,*) '99999.   99999.   0.'
      endif
      ncount=ncount+1
c  endif for icount
      endif

c  enddo for k
      enddo
c*******************************************************************************
      write(6,9),'A total of ',iraycount,' rays integrated.'
  9   format(1x,a,i5,a)
      print *
      print *,'Open the pod bay doors, please, Hal. '

      end

