      subroutine stps(s)
c
c  A routine to fill the vector "steps" of the common block table.h.
c
c  "steps" is used to define the step size, h, used for the Runge-Kutta integration.
c   Since we know the nature of the integration, predefined step sizes
c   are used, rather than adaptive stepping.  Adaptive stepping requires
c   a little more computation.
c
c  The linear to tanh form defined here is fairly arbitrary, but somewhat
c  mimics the step sizes determined empirically from an adaptive step size
c  calculation.  See Figure 1 of the Technical Report on eigenray for the results.
c
c  s(1):  depth for transition point from linear to tanh
c  s(2):  surface value for step size
c  s(3):  transition point value for step size
c  s(4):  abyssal value for step size
c
c  Brian Dushaw
c  Applied Physics Laboratory 
c  University of Washington
c  Seattle, WA 98105
c  (206) 543-1300
c  dushaw@apl.washington.edu
c
c  EIGENRAY Version 3.4
c  September-December 1998
c  Last updated November 2012

      implicit double precision (a-h,o-z)
      include 'nssp.h'
      include 'table.h'
      include 'constants.h'
      double precision s(4)

      do 64 jj=-1,Ndepmax
        ztemp=dble(jj)*depth_inc
        if (ztemp.lt.s(1)) then
c  linear above s(1)
          steps(jj)=((s(3)-s(2))/s(1))*ztemp + s(2)
        else
c  tanh below s(1)
          steps(jj)=s(3)+(s(4)-s(3))*dtanh((ztemp-s(1))/2000.d0)
        end if
64     continue

        return
        end
        
